/******************************************************************

	Command:  XPDL -- Explode Part of Dimension into Line

	Purpose:  This program is used to replace the extension
		  line of a dimension entity with a real line
		  for further trimming or breaking purpose.

	Programmed by Kang, Hsin-Min.
	Released in 1995 as a working example.
	Revised at Feb 09, 1998 -- Accept line entity for breaking
	Revised at Apr 22, 1998 Include Japenese Text in SJIS codes

	Copyright(c) 1995, 1998 TCAM Development House, Taipei.
	All rights Reserved.

	Status: TwinCAD Standard Command, with full support from
		TCAM Development House.

    Comments:

	The selected dimension's extension line nearest to
	the pick point will be suppressed and replaced with a
	real line entity.

*******************************************************************/
//#include "tclcad.h"
// Resolve definition from tclcad.h
#define T_LINE		2
#define T_DIM		9
#define FLINE		4
#define FDIM		0x200
#define DIMSE1		0x10	/* Suppress first extension line */
#define DIMSE2		0x20	/* Suppress second extension line */
#define OSINT		0x20	/* Intersection of two objects */
#define DIMLINE 	0
#define DIMANG		1

#define LoopForever	while(1)
#define nearp1(p1,p2,po)  (v(p1,po)<=v(p2,po))

#define TGBIG5		1
#define TGGB2312	2
#define TGSJIS		5
#define TGKOREAN	3

char	szText[5][70];		// Our display text buffer
char	szAsciiText[][70]={	// Default English Display Text
/*  0 */   "\nXPDL -- Please select dimension by its extension line: ",
/*  1 */   "\n** Please select linear or angular dimension.",
/*  2 */   "\nPlease indicate where to break: ",
/*  3 */   " -- %d lines produced."
};
char	szBig5Text[][60]={	// Default Chinese Big-5 Display Text
/*  0 */   "\nпث׼еϤu: ",
/*  1 */   "\n** пuʤث׼еΨפث׼еϤ.",
/*  2 */   "\nЫ_I: ",
/*  3 */   " --  %d u."
};
char	szGB2312Text[][70]={	// Default Chinese GB2312 Display Text
/*  0 */   "\nѡȡ߶ȱעͼԪ: ",
/*  1 */   "\n** ѡȡԳ߶ȱעǶȳ߶ȱעͼԪ.",
/*  2 */   "\nָʾ: ",
/*  3 */   " --  %d ߲."
};
//
//	The following Japenese messages are translated by _____________.
//
char	szSJISText[][70]={    // Default Japenese SJIS Display Text
/*  0 */   "\nXPDL -- @Iĉ: ",
/*  1 */   "\n** px̐@Iĉ",
/*  2 */   "\n ǂ𕪒f܂H ",
/*  3 */   " -- %d { 쐬܂B"
};
//
//	The following Korean messages are translated by _____________.
//
char	szKoreanText[][70]={	// Default Korean Display Text
/*  0 */   "\nXPDL -- Please select dimension by its extension line: ",
/*  1 */   "\n** Please select linear or angular dimension.",
/*  2 */   "\nPlease indicate where to break? ",
/*  3 */   " -- %d lines produced."
};

COMMAND XPDL()
{
	ENTDATA data;

	switch(@tgftype) {
	  case TGBIG5:	  setdata(szText,szBig5Text);	  break;
	  case TGGB2312:  setdata(szText,szGB2312Text);   break;
	  case TGSJIS:	  setdata(szText,szSJISText);	  break;
	  case TGKOREAN:  setdata(szText,szKoreanText);   break;
	  default:	  setdata(szText,szAsciiText);	  break;
	}

	LoopForever {
	    e1 = gete(szText[0],FDIM|FLINE);
	    if ( userbrk(-1) )	break;
	    if ( !ent_ok(e1) )	break;
	    if ( ent_type(e1)==T_DIM ) {
		data = ent_read(e1);
		switch(data.dim.type&0x0f) {
		  case DIMLINE:  XpLinear();	    continue;
		  case DIMANG:	 XpAngular();	    continue;
		}
	    }
	    if ( ent_type(e1)==T_LINE ) {
		XpLine();
		continue;
	    }
	    printf(szText[1]);
	}
}

XpLinear()
{
	ENTDATA data;

	data = ent_read(e1);

	if ( nearp1(data.dim.pds,data.dim.pde,e1.pick) ) {
	    if ( data.dim.flag & DIMSE1 )
		return;   // First Extension is already suppressed!
	    data.dim.flag |= DIMSE1;
	    l1.ps = data.dim.ps;
	    l1.pe = data.dim.pds;
	}
	else {
	    if ( data.dim.flag & DIMSE2 )
		return;   // Second Extension is already suppressed!
	    data.dim.flag |= DIMSE2;
	    l1.ps = data.dim.pe;
	    l1.pe = data.dim.pde;
	}

	@osmode = OSINT;
	p1 = getp(szText[2],l1.ps);
	@osmode = 0;

	ent_write(e1,data);

	l1.pe = l1,V(l1)+@dimexe;
	l1.ps = l1,@dimexo;

	CreateLine();
}

XpAngular()
{
	ENTDATA data;
	double	len;

	data = ent_read(e1);

	l1.ps = data.dim.ps;
	if ( nearp1(data.dim.pds,data.dim.pde,e1.pick) ) {
	    if ( data.dim.flag & DIMSE1 )
		return;   // First Extension is already suppressed!
	    data.dim.flag |= DIMSE1;
	    l1.pe = data.dim.pds;
	    len = data.dim.pe.x;
	}
	else {
	    if ( data.dim.flag & DIMSE2 )
		return;   // Second Extension is already suppressed!
	    data.dim.flag |= DIMSE2;
	    l1.pe = data.dim.pde;
	    len = data.dim.pe.y;
	}

	@osmode = OSINT;
	p1 = getp(szText[2],l1.ps);
	@osmode = 0;

	ent_write(e1,data);

	if ( len < V(data.dim.ps,l1.pe) ) {
	    l1.pe = l1,V(l1)+@dimexe;
	    l1.ps = l1,len+@dimexo;
	}
	else {
	    l1.pe = l1,V(l1)-@dimexe;
	    l1.ps = l1,len-@dimexo;
	}

	CreateLine();
}
/*
	L1 -- New line to create
	P1 -- Point to split the line
*/
CreateLine()
{
	ENTDATA data, data1;

	data = ent_read(e1);
	p1 = p1,l1;
	if ( (p1==l1.ps) || (p1==l1.pe) || !in_span(l1,p1) ) {
	    e2=newent(l1);
	    data1 = ent_read(e2);
	    data1.layer = data.layer;
	    data1.color = data.color;
	    data1.ltypeno = data.ltypeno;
	    ent_write(e2,data1);
	    printf(szText[3],1);
	}
	else {
	    l2.pe = L(p1,l1.ps),@dimgap;
	    l2.ps = l1.ps;
	    l1.ps = L(p1,l1.pe),@dimgap;

	    e2=newent(l1);
	    e3=newent(l2);

	    data1 = ent_read(e2);
	    data1.layer = data.layer;
	    data1.color = data.color;
	    data1.ltypeno = data.ltypeno;
	    ent_write(e2,data1);

	    data1 = ent_read(e3);
	    data1.layer = data.layer;
	    data1.color = data.color;
	    data1.ltypeno = data.ltypeno;
	    ent_write(e3,data1);
	    printf(szText[3],2);
	}
}

XpLine()
{
	ENTDATA data, data1;
	l1 = e1;
	@osmode = OSINT;
	p1 = getp(szText[2],l1.ps);
	@osmode = 0;
	data = ent_read(e1);
	p1 = p1,l1;
	if ( (p1==l1.ps) || (p1==l1.pe) || !in_span(l1,p1) ) {
	    return;
	}
	else {
	    l2.pe = L(p1,l1.ps),@dimgap;
	    l2.ps = l1.ps;
	    l1.ps = L(p1,l1.pe),@dimgap;

	    data.line.ps = l1.ps;
	    data.line.pe = l1.pe;
	    ent_write(e1,data);
	    e2=newent(l2);
	    data1 = ent_read(e2);
	    data1.layer = data.layer;
	    data1.color = data.color;
	    data1.ltypeno = data.ltypeno;
	    ent_write(e2,data1);
	}
}
